import pandas as pd
import numpy as np
from sqlalchemy import create_engine
from datetime import datetime, timedelta
from tqdm import tqdm
import matplotlib.pyplot as plt
import os

# v2版本只做表格

# 添加中文字体支持
plt.rcParams['font.sans-serif'] = ['SimHei']  # 用来正常显示中文标签
plt.rcParams['axes.unicode_minus'] = False  # 用来正常显示负号

class StockAnalyzer:
    def __init__(self, db_connection_string):
        """
        Initialize the stock analyzer
        :param db_connection_string: Database connection string
        """
        self.engine = create_engine(db_connection_string)
        self.initial_capital = 300000  # 30万本金
        self.total_position = 1000000  # 100万建仓规模
        self.borrowed_capital = 700000  # 70万借入资金
        self.annual_interest_rate = 0.05  # 年化5%利息
        self.daily_interest_rate = self.annual_interest_rate / 365
        self.commission_rate = 0.05  # 5%手续费
        self.min_holding_days = 7  # 最少持仓天数

    def get_stock_list(self):
        """获取所有股票列表"""
        # query = "SELECT DISTINCT gp_code as symbol FROM gp_code_all where mark1 = '1'"
        # return pd.read_sql(query, self.engine)['symbol'].tolist()
        return ['SH600522',
                'SZ002340',
                'SZ000733',
                'SH601615',
                'SH600157',
                'SH688005',
                'SZ002180',
                'SZ000723',
                'SZ002384',
                'SH600885',
                'SZ000009',
                'SZ300285',
                'SH605358',
                'SH601555',
                'SH601117',
                'SZ002385',
                'SH688099',
                'SH600884',
                'SH600298',
                'SH600399',
                'SZ002028',
                'SH603613',
                'SZ300037',
                'SH600765',
                'SH600256',
                'SH600487',
                'SH600563',
                'SH600754',
                'SZ002185',
                'SZ300363',
                'SH601636',
                'SZ300751',
                'SZ300088',
                'SH603589',
                'SZ000630',
                'SZ000988',
                'SZ002268',
                'SZ002353',
                'SH601689',
                'SH601077',
                'SZ002013',
                'SH601168',
                'SZ300763',
                'SH600141',
                'SH600392',
                'SZ000932',
                'SH603267',
                'SZ300699',
                'SH600233',
                'SZ000728',
                'SH600516',
                'SZ300058',
                'SH600486',
                'SZ002797',
                'SH603456',
                'SH603077',
                'SZ002138',
                'SH600862',
                'SZ300146',
                'SZ300223',
                'SZ300136',
                'SH600188',
                'SH600418',
                'SZ002673',
                'SH600705',
                'SH601128',
                'SZ000960',
                'SZ300308',
                'SH688065',
                'SZ300253',
                'SZ002080',
                'SZ000998',
                'SH688536',
                'SZ002092',
                'SH600875',
                'SZ000547',
                'SZ002409',
                'SH600482',
                'SH600521',
                'SZ000807',
                'SH600733',
                'SH600988',
                'SZ000039',
                'SZ300296',
                'SH600177',
                'SZ002439',
                'SZ002212',
                'SZ300418',
                'SZ002078',
                'SZ002223',
                'SH600739',
                'SH603893',
                'SZ002191',
                'SZ002030',
                'SH600699',
                'SZ002603',
                'SH600160',
                'SZ002444',
                'SH601198',
                'SZ300724',
                'SH600637',
                'SZ300474',
                'SZ002465',
                'SH688208',
                'SH603605',
                'SH688002',
                'SH600497',
                'SZ300383',
                'SZ002507',
                'SH601997',
                'SZ002557',
                'SH600549',
                'SZ000739',
                'SH600529',
                'SH600704',
                'SH603290',
                'SZ002508',
                'SH600038',
                'SZ002739',
                'SZ000636',
                'SH603198',
                'SZ002156',
                'SZ000830',
                'SH600201',
                'SH600153',
                'SH600369',
                'SZ000983',
                'SH600642',
                'SZ002299',
                'SZ000629',
                'SZ002936',
                'SZ002131',
                'SZ000738',
                'SH603127',
                'SH601016',
                'SZ000825',
                'SZ000623',
                'SH600879',
                'SH600859',
                'SZ000519',
                'SH600039',
                'SZ300618',
                'SH688521',
                'SZ002273',
                'SH603444',
                'SH600166',
                'SZ002422',
                'SH600732',
                'SZ000750',
                'SZ002373',
                'SH600803',
                'SH600170',
                'SZ000050',
                'SZ000887',
                'SZ300357',
                'SH601699',
                'SH600867',
                'SZ000656',
                'SH600118',
                'SZ300070',
                'SZ002372',
                'SZ000400',
                'SH600271',
                'SH600258',
                'SZ002926',
                'SZ300017',
                'SZ002153',
                'SZ002152',
                'SH600909',
                'SZ002195',
                'SZ300168',
                'SH600008',
                'SZ002572',
                'SH600315',
                'SZ002065',
                'SH600066',
                'SH600580',
                'SZ000975',
                'SH600027',
                'SZ000999',
                'SZ000997',
                'SH603218',
                'SH600536',
                'SZ002203',
                'SH601005',
                'SZ002985',
                'SH600316',
                'SH600372',
                'SH603707',
                'SZ000027',
                'SZ000513',
                'SZ300182',
                'SZ002506',
                'SH603156',
                'SZ002002',
                'SZ002408',
                'SZ002056',
                'SH600236',
                'SZ002266',
                'SZ300001',
                'SZ002500',
                'SZ300244',
                'SZ002250',
                'SH600801',
                'SZ002019',
                'SH600208',
                'SZ300024',
                'SH601872',
                'SH600998',
                'SZ300630',
                'SZ002221',
                'SZ002249',
                'SZ000021',
                'SH600348',
                'SZ300251',
                'SH600415',
                'SH600535',
                'SZ000401',
                'SZ000878',
                'SZ002831',
                'SZ000012',
                'SZ300009',
                'SH600728',
                'SZ000686',
                'SZ002430',
                'SH600171',
                'SZ000537',
                'SZ300115',
                'SZ002683',
                'SH601866',
                'SZ002745',
                'SZ002511',
                'SZ000060',
                'SH600718',
                'SH603568',
                'SZ002127',
                'SZ002532',
                'SH600390',
                'SH601992',
                'SZ300482',
                'SZ000970',
                'SH603858',
                'SH600839',
                'SH600970',
                'SZ002966',
                'SH600155',
                'SZ002128',
                'SZ000690',
                'SH600060',
                'SH600282',
                'SH600517',
                'SH600985',
                'SZ300463',
                'SH600498',
                'SZ300026',
                'SH600380',
                'SH600673',
                'SZ000778',
                'SH603712',
                'SH600528',
                'SZ300212',
                'SZ300072',
                'SZ000930',
                'SH600325',
                'SH600667',
                'SH600782',
                'SZ002382',
                'SZ000729',
                'SZ002939',
                'SH600895',
                'SH600511',
                'SH603885',
                'SH688006',
                'SZ002294',
                'SZ000540',
                'SH601456',
                'SZ000581',
                'SH600500',
                'SH600863',
                'SH601717',
                'SZ002505',
                'SZ002958',
                'SH601179',
                'SZ300315',
                'SH600216',
                'SH600755',
                'SZ002396',
                'SZ000598',
                'SH600297',
                'SZ002701',
                'SH600820',
                'SH600567',
                'SZ000709',
                'SH603927',
                'SH603638',
                'SH603225',
                'SZ002670',
                'SH600507',
                'SH603866',
                'SZ002595',
                'SZ002004',
                'SZ002075',
                'SZ002368',
                'SZ300376',
                'SZ002146',
                'SZ002048',
                'SH600409',
                'SH600021',
                'SH600446',
                'SH601778',
                'SZ300271',
                'SH601577',
                'SH600827',
                'SH688188',
                'SH601880',
                'SH600131',
                'SH600598',
                'SH600808',
                'SZ002010',
                'SZ002925',
                'SH600663',
                'SZ300166',
                'SH600022',
                'SZ002124',
                'SH600546',
                'SH600764',
                'SZ002690',
                'SZ000987',
                'SZ002174',
                'SH600597',
                'SH600737',
                'SH600908',
                'SH603883',
                'SH603228',
                'SZ000883',
                'SZ002155',
                'SH600811',
                'SH601106',
                'SZ002233',
                'SZ002110',
                'SH600566',
                'SZ002281',
                'SH600158',
                'SH600967',
                'SZ000898',
                'SH601118',
                'SH600707',
                'SH600572',
                'SZ002440',
                'SZ000488',
                'SZ000961',
                'SH688029',
                'SZ000990',
                'SZ002183',
                'SH600435',
                'SH601000',
                'SZ000090',
                'SZ000877',
                'SH600906',
                'SH600064',
                'SH601333',
                'SZ002429',
                'SH688088',
                'SH600398',
                'SH601200',
                'SZ002081',
                'SZ000089',
                'SH600373',
                'SH600037',
                'SZ002867',
                'SH600376',
                'SZ001914',
                'SZ002085',
                'SH600167',
                'SZ000559',
                'SZ000959',
                'SZ000528',
                'SZ002416',
                'SZ002244',
                'SH600195',
                'SH600329',
                'SH600259',
                'SZ002434',
                'SH688321',
                'SZ000685',
                'SH601718',
                'SZ000671',
                'SH600026',
                'SH603000',
                'SZ002038',
                'SH600120',
                'SZ002242',
                'SZ001965',
                'SH601608',
                'SZ300133',
                'SH600126',
                'SZ002705',
                'SZ000402',
                'SZ000158',
                'SZ002390',
                'SZ002387',
                'SH601611',
                'SZ000758',
                'SH601098',
                'SZ000415',
                'SH600307',
                'SH601991',
                'SH601975',
                'SH601598',
                'SH603650',
                'SH600959',
                'SH688289',
                'SH600582',
                'SH600643',
                'SZ000563',
                'SZ000718',
                'SH600062',
                'SZ300257',
                'SH601958',
                'SH600901',
                'SH600266',
                'SH600717',
                'SZ000967',
                'SZ002399',
                'SH600056',
                'SZ002458',
                'SH601860',
                'SZ002563',
                'SZ003022',
                'SH601928',
                'SH600928',
                'SH600729',
                'SZ002423',
                'SH600968',
                'SZ000937',
                'SZ002468',
                'SZ000717',
                'SH600556',
                'SZ003035',
                'SH600649',
                'SH600787',
                'SZ000028',
                'SH600095',
                'SH600776',
                'SZ000031',
                'SH600339',
                'SH600006',
                'SH603317',
                'SH600648',
                'SH600623',
                'SZ000062',
                'SH600835',
                'SZ002815',
                'SZ002424',
                'SH600377',
                'SH601139',
                'SH600299',
                'SZ002948',
                'SZ000156',
                'SZ002375',
                'SZ000869',
                'SH600871',
                'SH601828',
                'SZ300869',
                'SZ002302',
                'SH601228',
                'SH601969',
                'SH600639',
                'SH601298',
                'SH603379',
                'SZ000046',
                'SZ002945',
                'SZ002653',
                'SZ001872',
                'SH601156',
                'SH600823',
                'SZ300741',
                'SH603056',
                'SH601869',
                'SH603786',
                'SH603708',
                'SZ000553',
                'SH600350',
                'SZ001203',
                'SH600657',
                'SH601568',
                'SH600466',
                'SH603515',
                'SH603355',
                'SH603719',
                'SH601665',
                'SH601003',
                'SZ002946',
                'SH600917',
                'SH603868',
                'SZ002901',
                'SH600903',
                'SH600956',
                'SH601187',
                'SH603983']  # 你可以在这里添加更多股票代码

    def get_stock_data(self, symbol, start_date, end_date, include_history=False):
        """获取指定股票在时间范围内的数据"""
        try:
            if include_history:
                # 如果需要历史数据，获取start_date之前240天的数据
                query = f"""
                WITH history_data AS (
                    SELECT 
                        symbol,
                        timestamp,
                        CAST(close as DECIMAL(10,2)) as close
                    FROM gp_day_data 
                    WHERE symbol = '{symbol}' 
                    AND timestamp < '{start_date}'
                    ORDER BY timestamp DESC
                    LIMIT 240
                )
                SELECT 
                    symbol,
                    timestamp,
                    CAST(open as DECIMAL(10,2)) as open,
                    CAST(high as DECIMAL(10,2)) as high,
                    CAST(low as DECIMAL(10,2)) as low,
                    CAST(close as DECIMAL(10,2)) as close,
                    CAST(volume as DECIMAL(20,0)) as volume
                FROM gp_day_data 
                WHERE symbol = '{symbol}' 
                AND timestamp BETWEEN '{start_date}' AND '{end_date}'
                UNION ALL
                SELECT 
                    symbol,
                    timestamp,
                    NULL as open,
                    NULL as high,
                    NULL as low,
                    close,
                    NULL as volume
                FROM history_data
                ORDER BY timestamp
                """
            else:
                query = f"""
                SELECT 
                    symbol,
                    timestamp,
                    CAST(open as DECIMAL(10,2)) as open,
                    CAST(high as DECIMAL(10,2)) as high,
                    CAST(low as DECIMAL(10,2)) as low,
                    CAST(close as DECIMAL(10,2)) as close,
                    CAST(volume as DECIMAL(20,0)) as volume
                FROM gp_day_data 
                WHERE symbol = '{symbol}' 
                AND timestamp BETWEEN '{start_date}' AND '{end_date}'
                ORDER BY timestamp
                """
            
            # 使用chunksize分批读取数据
            chunks = []
            for chunk in pd.read_sql(query, self.engine, chunksize=1000):
                chunks.append(chunk)
            df = pd.concat(chunks, ignore_index=True)
            
            # 确保数值列的类型正确
            numeric_columns = ['open', 'high', 'low', 'close', 'volume']
            for col in numeric_columns:
                if col in df.columns:
                    df[col] = pd.to_numeric(df[col], errors='coerce')
            
            # 删除任何包含 NaN 的行，但只考虑分析期间需要的列
            if include_history:
                historical_mask = (df['timestamp'] < start_date) & df['close'].notna()
                analysis_mask = (df['timestamp'] >= start_date) & df[['open', 'high', 'low', 'close', 'volume']].notna().all(axis=1)
                df = df[historical_mask | analysis_mask]
            else:
                df = df.dropna()
            
            return df
            
        except Exception as e:
            print(f"获取股票 {symbol} 数据时出错: {str(e)}")
            return pd.DataFrame()

    def calculate_trade_signals(self, df, take_profit_pct, stop_loss_pct, pullback_entry_pct, start_date):
        """计算交易信号和收益"""
        """计算交易信号和收益"""
        # 检查历史数据是否足够
        historical_data = df[df['timestamp'] < start_date]
        if len(historical_data) < 240:
            # print(f"历史数据不足240天，跳过分析")
            return pd.DataFrame()

        # 计算240日均线基准价格（使用历史数据）
        ma240_base_price = historical_data['close'].tail(240).mean()
        # print(f"240日均线基准价格: {ma240_base_price:.2f}")

        # 只使用分析期间的数据进行交易信号计算
        analysis_df = df[df['timestamp'] >= start_date].copy()
        if len(analysis_df) < 10:  # 确保分析期间有足够的数据
            # print(f"分析期间数据不足10天，跳过分析")
            return pd.DataFrame()

        results = []
        positions = []  # 记录所有持仓，每个元素是一个字典，包含入场价格和日期
        max_positions = 5  # 最大建仓次数
        cumulative_profit = 0  # 总累计收益
        trades_count = 0
        max_holding_period = timedelta(days=180)  # 最长持仓6个月
        max_loss_amount = -300000  # 最大亏损金额为30万
        
        # 添加统计变量
        total_holding_days = 0  # 总持仓天数
        max_capital_usage = 0  # 最大资金占用
        total_trades = 0  # 总交易次数
        profitable_trades = 0  # 盈利交易次数
        loss_trades = 0  # 亏损交易次数

        # print(f"\n{'='*50}")
        # print(f"开始分析股票: {df.iloc[0]['symbol']}")
        # print(f"{'='*50}")

        # 记录前一天的收盘价
        prev_close = None

        for index, row in analysis_df.iterrows():
            current_price = (float(row['high']) + float(row['low'])) / 2
            day_result = {
                'timestamp': row['timestamp'],
                'profit': cumulative_profit,  # 记录当前的累计收益
                'position': False,
                'action': 'hold'
            }

            # 更新所有持仓的收益
            positions_to_remove = []

            for pos in positions:
                pos_days_held = (row['timestamp'] - pos['entry_date']).days
                pos_size = self.total_position / pos['entry_price']
                
                # 检查是否需要平仓
                should_close = False
                close_type = None
                close_price = current_price

                if pos_days_held >= self.min_holding_days:
                    # 检查止盈
                    if float(row['high']) >= pos['entry_price'] * (1 + take_profit_pct):
                        should_close = True
                        close_type = 'sell_profit'
                        close_price = pos['entry_price'] * (1 + take_profit_pct)
                    # 检查最大亏损
                    elif (close_price - pos['entry_price']) * pos_size <= max_loss_amount:
                        should_close = True
                        close_type = 'sell_loss'
                    # 检查持仓时间
                    elif (row['timestamp'] - pos['entry_date']) >= max_holding_period:
                        should_close = True
                        close_type = 'time_limit'

                if should_close:
                    profit = (close_price - pos['entry_price']) * pos_size
                    commission = profit * self.commission_rate if profit > 0 else 0
                    interest_cost = self.borrowed_capital * self.daily_interest_rate * pos_days_held
                    net_profit = profit - commission - interest_cost
                    
                    # 更新统计数据
                    total_holding_days += pos_days_held
                    total_trades += 1
                    if net_profit > 0:
                        profitable_trades += 1
                    else:
                        loss_trades += 1
                    
                    cumulative_profit += net_profit
                    positions_to_remove.append(pos)
                    day_result['action'] = close_type
                    day_result['profit'] = cumulative_profit
                    
                    # print(f"\n>>> {'止盈平仓' if close_type == 'sell_profit' else '止损平仓' if close_type == 'sell_loss' else '到期平仓'}")
                    # print(f"持仓天数: {pos_days_held}")
                    # print(f"持仓数量: {pos_size:.0f}股")
                    # print(f"平仓价格: {close_price:.2f}")
                    # print(f"{'盈利' if profit > 0 else '亏损'}: {profit:,.2f}")
                    # print(f"手续费: {commission:,.2f}")
                    # print(f"利息成本: {interest_cost:,.2f}")
                    # print(f"净{'盈利' if net_profit > 0 else '亏损'}: {net_profit:,.2f}")

            # 移除已平仓的持仓
            for pos in positions_to_remove:
                positions.remove(pos)

            # 检查是否可以建仓
            if len(positions) < max_positions and prev_close is not None:
                # 计算当日跌幅
                daily_drop = (prev_close - float(row['low'])) / prev_close
                
                # 检查价格是否在均线区间内
                price_in_range = (
                    current_price >= ma240_base_price * 0.7 and 
                    current_price <= ma240_base_price * 1.3
                )
                
                # 如果跌幅超过5%且价格在均线区间内
                if daily_drop >= 0.05 and price_in_range:
                    positions.append({
                        'entry_price': current_price,
                        'entry_date': row['timestamp']
                    })
                    trades_count += 1
                    # print(f"\n>>> 建仓信号 #{trades_count}")
                    # print(f"日期: {row['timestamp'].strftime('%Y-%m-%d')}")
                    # print(f"建仓价格: {current_price:.2f}")
                    # print(f"建仓数量: {self.total_position/current_price:.0f}股")
                    # print(f"当日跌幅: {daily_drop*100:.2f}%")
                    # print(f"距离均线: {((current_price/ma240_base_price)-1)*100:.2f}%")

            # 更新前一天收盘价
            prev_close = float(row['close'])

            # 更新日结果
            day_result['position'] = len(positions) > 0
            results.append(day_result)

            # 计算当前资金占用
            current_capital_usage = sum(self.total_position for pos in positions)
            max_capital_usage = max(max_capital_usage, current_capital_usage)

        # 在最后一天强制平仓所有持仓
        if positions:
            final_price = (float(analysis_df.iloc[-1]['high']) + float(analysis_df.iloc[-1]['low'])) / 2
            final_total_profit = 0
            
            for pos in positions:
                position_size = self.total_position / pos['entry_price']
                days_held = (analysis_df.iloc[-1]['timestamp'] - pos['entry_date']).days
                final_profit = (final_price - pos['entry_price']) * position_size
                commission = final_profit * self.commission_rate if final_profit > 0 else 0
                interest_cost = self.borrowed_capital * self.daily_interest_rate * days_held
                net_profit = final_profit - commission - interest_cost
                
                # 更新统计数据
                total_holding_days += days_held
                total_trades += 1
                if net_profit > 0:
                    profitable_trades += 1
                else:
                    loss_trades += 1
                
                final_total_profit += net_profit
                
                # print(f"\n>>> 到期强制平仓")
                # print(f"持仓天数: {days_held}")
                # print(f"持仓数量: {position_size:.0f}股")
                # print(f"平仓价格: {final_price:.2f}")
                # print(f"毛利润: {final_profit:,.2f}")
                # print(f"手续费: {commission:,.2f}")
                # print(f"利息成本: {interest_cost:,.2f}")
                # print(f"净利润: {net_profit:,.2f}")
            
            results[-1]['action'] = 'final_sell'
            cumulative_profit += final_total_profit  # 更新最终的累计收益
            results[-1]['profit'] = cumulative_profit  # 更新最后一天的累计收益

        # 计算统计数据
        avg_holding_days = total_holding_days / total_trades if total_trades > 0 else 0
        win_rate = profitable_trades / total_trades * 100 if total_trades > 0 else 0

        # print(f"\n{'='*50}")
        # print(f"交易统计")
        # print(f"总交易次数: {trades_count}")
        # print(f"累计收益: {cumulative_profit:,.2f}")
        # print(f"最大资金占用: {max_capital_usage:,.2f}")
        # print(f"平均持仓天数: {avg_holding_days:.1f}")
        # print(f"胜率: {win_rate:.1f}%")
        # print(f"盈利交易: {profitable_trades}次")
        # print(f"亏损交易: {loss_trades}次")
        # print(f"{'='*50}\n")

        # 将统计数据添加到结果中
        if len(results) > 0:
            results[-1]['stats'] = {
                'total_trades': total_trades,
                'profitable_trades': profitable_trades,
                'loss_trades': loss_trades,
                'win_rate': win_rate,
                'avg_holding_days': avg_holding_days,
                'max_capital_usage': max_capital_usage,
                'final_profit': cumulative_profit
            }

        return pd.DataFrame(results)

    def analyze_stock(self, symbol, start_date, end_date, take_profit_pct, stop_loss_pct, pullback_entry_pct):
        """分析单个股票"""
        df = self.get_stock_data(symbol, start_date, end_date, include_history=True)
        
        if len(df) < 240:  # 确保总数据量足够
            print(f"股票 {symbol} 总数据不足240天，跳过分析")
            return None
            
        return self.calculate_trade_signals(
            df,
            take_profit_pct=take_profit_pct,
            stop_loss_pct=stop_loss_pct,
            pullback_entry_pct=pullback_entry_pct,
            start_date=start_date
        )

    def analyze_all_stocks(self, start_date, end_date, take_profit_pct, stop_loss_pct, pullback_entry_pct):
        """分析所有股票"""
        stocks = self.get_stock_list()
        all_results = {}
        analysis_summary = {
            'processed': [],
            'skipped': [],
            'error': []
        }
        
        print(f"\n开始分析，共 {len(stocks)} 支股票")
        
        for symbol in tqdm(stocks, desc="Analyzing stocks"):
            try:
                # 获取股票数据
                df = self.get_stock_data(symbol, start_date, end_date, include_history=True)
                
                if df.empty:
                    print(f"\n股票 {symbol} 数据为空，跳过分析")
                    analysis_summary['skipped'].append({
                        'symbol': symbol,
                        'reason': 'empty_data',
                        'take_profit_pct': take_profit_pct
                    })
                    continue
                
                # 检查历史数据
                historical_data = df[df['timestamp'] < start_date]
                if len(historical_data) < 240:
                    # print(f"\n股票 {symbol} 历史数据不足240天({len(historical_data)}天)，跳过分析")
                    analysis_summary['skipped'].append({
                        'symbol': symbol,
                        'reason': f'insufficient_history_{len(historical_data)}days',
                        'take_profit_pct': take_profit_pct
                    })
                    continue
                
                # 检查分析期数据
                analysis_data = df[df['timestamp'] >= start_date]
                if len(analysis_data) < 10:
                    print(f"\n股票 {symbol} 分析期数据不足10天({len(analysis_data)}天)，跳过分析")
                    analysis_summary['skipped'].append({
                        'symbol': symbol,
                        'reason': f'insufficient_analysis_data_{len(analysis_data)}days',
                        'take_profit_pct': take_profit_pct
                    })
                    continue
                
                # 计算交易信号
                result = self.calculate_trade_signals(
                    df,
                    take_profit_pct,
                    stop_loss_pct,
                    pullback_entry_pct,
                    start_date
                )
                
                if result is not None and not result.empty:
                    # 检查是否有交易记录
                    has_trades = any(action in ['sell_profit', 'sell_loss', 'time_limit', 'final_sell'] 
                                   for action in result['action'] if pd.notna(action))
                    
                    if has_trades:
                        all_results[symbol] = result
                        analysis_summary['processed'].append({
                            'symbol': symbol,
                            'take_profit_pct': take_profit_pct,
                            'trades': len([x for x in result['action'] if pd.notna(x) and x != 'hold']),
                            'profit': result['profit'].iloc[-1]
                        })
                    else:
                        # print(f"\n股票 {symbol} 没有产生有效的交易信号")
                        analysis_summary['skipped'].append({
                            'symbol': symbol,
                            'reason': 'no_valid_signals',
                            'take_profit_pct': take_profit_pct
                        })
                
            except Exception as e:
                print(f"\n处理股票 {symbol} 时出错: {str(e)}")
                analysis_summary['error'].append({
                    'symbol': symbol,
                    'error': str(e),
                    'take_profit_pct': take_profit_pct
                })
                continue
        
        # 打印分析总结
        print(f"\n分析完成:")
        print(f"成功分析: {len(analysis_summary['processed'])} 支股票")
        print(f"跳过分析: {len(analysis_summary['skipped'])} 支股票")
        print(f"出错股票: {len(analysis_summary['error'])} 支股票")
        
        # 保存分析总结
        if not os.path.exists('results/analysis_summary'):
            os.makedirs('results/analysis_summary')
        
        # 保存处理成功的股票信息
        if analysis_summary['processed']:
            pd.DataFrame(analysis_summary['processed']).to_csv(
                f'results/analysis_summary/processed_stocks_{take_profit_pct*100:.1f}.csv',
                index=False
            )
        
        # 保存跳过的股票信息
        if analysis_summary['skipped']:
            pd.DataFrame(analysis_summary['skipped']).to_csv(
                f'results/analysis_summary/skipped_stocks_{take_profit_pct*100:.1f}.csv',
                index=False
            )
        
        # 保存出错的股票信息
        if analysis_summary['error']:
            pd.DataFrame(analysis_summary['error']).to_csv(
                f'results/analysis_summary/error_stocks_{take_profit_pct*100:.1f}.csv',
                index=False
            )
        
        return all_results

    def plot_results(self, results, symbol):
        """绘制单个股票的收益走势图"""
        plt.figure(figsize=(12, 6))
        plt.plot(results['timestamp'], results['profit'].cumsum(), label='Cumulative Profit')
        plt.title(f'Stock {symbol} Trading Results')
        plt.xlabel('Date')
        plt.ylabel('Profit (CNY)')
        plt.legend()
        plt.grid(True)
        plt.xticks(rotation=45)
        plt.tight_layout()
        return plt

    def plot_all_stocks(self, all_results):
        """绘制所有股票的收益走势图"""
        plt.figure(figsize=(15, 8))
        
        # 记录所有股票的累计收益
        total_profits = {}
        max_profit = float('-inf')
        min_profit = float('inf')
        
        # 绘制每支股票的曲线
        for symbol, results in all_results.items():
            # 直接使用已经计算好的累计收益
            profits = results['profit']
            plt.plot(results['timestamp'], profits, label=symbol, alpha=0.5, linewidth=1)
            
            # 更新最大最小收益
            max_profit = max(max_profit, profits.max())
            min_profit = min(min_profit, profits.min())
            
            # 记录总收益（使用最后一个累计收益值）
            total_profits[symbol] = profits.iloc[-1]
        
        # 计算所有股票的中位数收益曲线
        # 首先找到共同的日期范围
        all_dates = set()
        for results in all_results.values():
            all_dates.update(results['timestamp'])
        all_dates = sorted(list(all_dates))
        
        # 创建一个包含所有日期的DataFrame
        profits_df = pd.DataFrame(index=all_dates)
        
        # 对每支股票，填充所有日期的收益
        for symbol, results in all_results.items():
            daily_profits = pd.Series(index=all_dates, data=np.nan)
            # 直接使用已经计算好的累计收益
            for date, profit in zip(results['timestamp'], results['profit']):
                daily_profits[date] = profit
            # 对于没有交易的日期，使用最后一个累计收益填充
            daily_profits.fillna(method='ffill', inplace=True)
            daily_profits.fillna(0, inplace=True)  # 对开始之前的日期填充0
            profits_df[symbol] = daily_profits
        
        # 计算并绘制中位数收益曲线
        median_line = profits_df.median(axis=1)
        plt.plot(all_dates, median_line, 'r-', label='Median', linewidth=2)
        
        plt.title('All Stocks Trading Results')
        plt.xlabel('Date')
        plt.ylabel('Profit (CNY)')
        plt.grid(True)
        plt.xticks(rotation=45)
        
        # 添加利润区间标注
        plt.text(0.02, 0.98, 
                f'Profit Range:\nMax: {max_profit:,.0f}\nMin: {min_profit:,.0f}\nMedian: {median_line.iloc[-1]:,.0f}', 
                transform=plt.gca().transAxes, 
                bbox=dict(facecolor='white', alpha=0.8))
        
        # 计算所有股票的统计数据
        total_stats = {
            'total_trades': 0,
            'profitable_trades': 0,
            'loss_trades': 0,
            'total_holding_days': 0,
            'max_capital_usage': 0,
            'total_profit': 0,
            'profitable_stocks': 0,
            'loss_stocks': 0,
            'total_capital_usage': 0,  # 添加总资金占用
            'total_win_rate': 0,       # 添加总胜率
        }

        # 获取前5名和后5名股票
        top_5 = sorted(total_profits.items(), key=lambda x: x[1], reverse=True)[:5]
        bottom_5 = sorted(total_profits.items(), key=lambda x: x[1])[:5]
        
        # 添加排名信息
        rank_text = "Top 5 Stocks:\n"
        for symbol, profit in top_5:
            rank_text += f"{symbol}: {profit:,.0f}\n"
        rank_text += "\nBottom 5 Stocks:\n"
        for symbol, profit in bottom_5:
            rank_text += f"{symbol}: {profit:,.0f}\n"
            
        plt.text(1.02, 0.98, rank_text, transform=plt.gca().transAxes, 
                bbox=dict(facecolor='white', alpha=0.8), verticalalignment='top')

        # 记录每支股票的统计数据用于计算平均值
        stock_stats = []

        for symbol, results in all_results.items():
            if 'stats' in results.iloc[-1]:
                stats = results.iloc[-1]['stats']
                stock_stats.append(stats)  # 记录每支股票的统计数据
                
                total_stats['total_trades'] += stats['total_trades']
                total_stats['profitable_trades'] += stats['profitable_trades']
                total_stats['loss_trades'] += stats['loss_trades']
                total_stats['total_holding_days'] += stats['avg_holding_days'] * stats['total_trades']
                total_stats['max_capital_usage'] = max(total_stats['max_capital_usage'], stats['max_capital_usage'])
                total_stats['total_capital_usage'] += stats['max_capital_usage']
                total_stats['total_profit'] += stats['final_profit']
                total_stats['total_win_rate'] += stats['win_rate']
                if stats['final_profit'] > 0:
                    total_stats['profitable_stocks'] += 1
                else:
                    total_stats['loss_stocks'] += 1

        # 计算总体统计
        total_stocks = total_stats['profitable_stocks'] + total_stats['loss_stocks']
        avg_holding_days = total_stats['total_holding_days'] / total_stats['total_trades'] if total_stats['total_trades'] > 0 else 0
        win_rate = total_stats['profitable_trades'] / total_stats['total_trades'] * 100 if total_stats['total_trades'] > 0 else 0
        stock_win_rate = total_stats['profitable_stocks'] / total_stocks * 100 if total_stocks > 0 else 0
        
        # 计算平均值统计
        avg_trades_per_stock = total_stats['total_trades'] / total_stocks if total_stocks > 0 else 0
        avg_profit_per_stock = total_stats['total_profit'] / total_stocks if total_stocks > 0 else 0
        avg_capital_usage = total_stats['total_capital_usage'] / total_stocks if total_stocks > 0 else 0
        avg_win_rate = total_stats['total_win_rate'] / total_stocks if total_stocks > 0 else 0

        # 添加总体统计信息
        stats_text = (
            f"总体统计:\n"
            f"总交易次数: {total_stats['total_trades']}\n"
            # f"最大资金占用: {total_stats['max_capital_usage']:,.0f}\n"
            f"平均持仓天数: {avg_holding_days:.1f}\n"
            f"交易胜率: {win_rate:.1f}%\n"
            f"股票胜率: {stock_win_rate:.1f}%\n"
            f"盈利股票数: {total_stats['profitable_stocks']}\n"
            f"亏损股票数: {total_stats['loss_stocks']}\n"
            f"\n平均值统计:\n"
            f"平均交易次数: {avg_trades_per_stock:.1f}\n"
            f"平均收益: {avg_profit_per_stock:,.0f}\n"
            f"平均资金占用: {avg_capital_usage:,.0f}\n"
            f"平均持仓天数: {avg_holding_days:.1f}\n"
            f"平均胜率: {avg_win_rate:.1f}%"
        )
        
        plt.text(0.02, 0.6, stats_text,
                transform=plt.gca().transAxes,
                bbox=dict(facecolor='white', alpha=0.8, edgecolor='gray'),
                verticalalignment='top',
                fontsize=10)
        
        # 获取前5名和后5名股票
        top_5 = sorted(total_profits.items(), key=lambda x: x[1], reverse=True)[:5]
        bottom_5 = sorted(total_profits.items(), key=lambda x: x[1])[:5]
        
        # 添加排名信息
        rank_text = "Top 5 Stocks:\n"
        for symbol, profit in top_5:
            rank_text += f"{symbol}: {profit:,.0f}\n"
        rank_text += "\nBottom 5 Stocks:\n"
        for symbol, profit in bottom_5:
            rank_text += f"{symbol}: {profit:,.0f}\n"
            
        plt.text(1.02, 0.98, rank_text, transform=plt.gca().transAxes, 
                bbox=dict(facecolor='white', alpha=0.8), verticalalignment='top')
        
        plt.tight_layout()
        return plt, total_profits

    def plot_profit_distribution(self, total_profits):
        """绘制所有股票的收益分布小提琴图"""
        plt.figure(figsize=(12, 8))
        
        # 将收益数据转换为数组，确保使用最终累计收益
        profits = np.array([profit for profit in total_profits.values()])
        
        # 计算统计数据
        mean_profit = np.mean(profits)
        median_profit = np.median(profits)
        max_profit = np.max(profits)
        min_profit = np.min(profits)
        std_profit = np.std(profits)
        
        # 绘制小提琴图
        violin_parts = plt.violinplot(profits, positions=[0], showmeans=True, showmedians=True)
        
        # 设置小提琴图的颜色
        violin_parts['bodies'][0].set_facecolor('lightblue')
        violin_parts['bodies'][0].set_alpha(0.7)
        violin_parts['cmeans'].set_color('red')
        violin_parts['cmedians'].set_color('blue')
        
        # 添加统计信息
        stats_text = (
            f"统计信息:\n"
            f"平均收益: {mean_profit:,.0f}\n"
            f"中位数收益: {median_profit:,.0f}\n"
            f"最大收益: {max_profit:,.0f}\n"
            f"最小收益: {min_profit:,.0f}\n"
            f"标准差: {std_profit:,.0f}"
        )
        
        plt.text(0.65, 0.95, stats_text,
                transform=plt.gca().transAxes,
                bbox=dict(facecolor='white', alpha=0.8, edgecolor='gray'),
                verticalalignment='top',
                fontsize=10)
        
        # 设置图表样式
        plt.title('股票收益分布图', fontsize=14, pad=20)
        plt.ylabel('收益 (元)', fontsize=12)
        plt.xticks([0], ['所有股票'], fontsize=10)
        plt.grid(True, axis='y', alpha=0.3)
        
        # 添加零线
        plt.axhline(y=0, color='r', linestyle='--', alpha=0.3)
        
        # 计算盈利和亏损的股票数量
        profit_count = np.sum(profits > 0)
        loss_count = np.sum(profits < 0)
        total_count = len(profits)
        
        # 添加盈亏比例信息
        ratio_text = (
            f"盈亏比例:\n"
            f"盈利: {profit_count}支 ({profit_count/total_count*100:.1f}%)\n"
            f"亏损: {loss_count}支 ({loss_count/total_count*100:.1f}%)\n"
            f"总计: {total_count}支"
        )
        
        plt.text(0.65, 0.6, ratio_text,
                transform=plt.gca().transAxes,
                bbox=dict(facecolor='white', alpha=0.8, edgecolor='gray'),
                verticalalignment='top',
                fontsize=10)
        
        plt.tight_layout()
        return plt

    def plot_profit_matrix(self, df):
        """绘制止盈比例分析矩阵的结果图"""
        plt.figure(figsize=(15, 10))
        
        # 创建子图
        fig, ((ax1, ax2), (ax3, ax4)) = plt.subplots(2, 2, figsize=(15, 10))
        
        # 1. 总收益曲线
        ax1.plot(df.index, df['total_profit'], 'b-', linewidth=2)
        ax1.set_title('总收益 vs 止盈比例')
        ax1.set_xlabel('止盈比例 (%)')
        ax1.set_ylabel('总收益 (元)')
        ax1.grid(True)
        
        # 标记最优止盈比例点
        best_profit_pct = df['total_profit'].idxmax()
        best_profit = df['total_profit'].max()
        ax1.plot(best_profit_pct, best_profit, 'ro')
        ax1.annotate(f'最优: {best_profit_pct:.1f}%\n{best_profit:,.0f}元',
                    (best_profit_pct, best_profit),
                    xytext=(10, 10), textcoords='offset points')
        
        # 2. 平均每笔收益和中位数收益
        ax2.plot(df.index, df['avg_profit_per_trade'], 'g-', linewidth=2, label='平均收益')
        ax2.plot(df.index, df['median_profit_per_trade'], 'r--', linewidth=2, label='中位数收益')
        ax2.set_title('每笔交易收益 vs 止盈比例')
        ax2.set_xlabel('止盈比例 (%)')
        ax2.set_ylabel('收益 (元)')
        ax2.legend()
        ax2.grid(True)
        
        # 3. 平均持仓天数曲线
        ax3.plot(df.index, df['avg_holding_days'], 'r-', linewidth=2)
        ax3.set_title('平均持仓天数 vs 止盈比例')
        ax3.set_xlabel('止盈比例 (%)')
        ax3.set_ylabel('持仓天数')
        ax3.grid(True)
        
        # 4. 胜率对比
        ax4.plot(df.index, df['trade_win_rate'], 'c-', linewidth=2, label='交易胜率')
        ax4.plot(df.index, df['stock_win_rate'], 'm--', linewidth=2, label='股票胜率')
        ax4.set_title('胜率 vs 止盈比例')
        ax4.set_xlabel('止盈比例 (%)')
        ax4.set_ylabel('胜率 (%)')
        ax4.legend()
        ax4.grid(True)
        
        # 调整布局
        plt.tight_layout()
        return plt

    def analyze_profit_matrix(self, start_date, end_date, stop_loss_pct, pullback_entry_pct):
        """分析不同止盈比例的表现矩阵"""
        # 创建results目录和临时文件目录
        if not os.path.exists('results/temp'):
            os.makedirs('results/temp')
            
        # 从30%到5%，每次减少1%
        profit_percentages = list(np.arange(0.30, 0.04, -0.01))
        
        for take_profit_pct in profit_percentages:
            # 检查是否已经分析过这个止盈点
            temp_file = f'results/temp/profit_{take_profit_pct*100:.1f}.csv'
            if os.path.exists(temp_file):
                print(f"\n止盈比例 {take_profit_pct*100:.1f}% 已分析，跳过")
                continue
                
            print(f"\n分析止盈比例: {take_profit_pct*100:.1f}%")
            
            try:
                # 运行分析
                results = self.analyze_all_stocks(
                    start_date,
                    end_date,
                    take_profit_pct,
                    stop_loss_pct,
                    pullback_entry_pct
                )
                
                # 过滤掉空结果
                valid_results = {symbol: result for symbol, result in results.items() if result is not None and not result.empty}
                
                if valid_results:
                    # 计算统计数据
                    total_stats = {
                        'total_trades': 0,
                        'profitable_trades': 0,
                        'loss_trades': 0,
                        'total_holding_days': 0,
                        'total_profit': 0,
                        'total_capital_usage': 0,
                        'total_win_rate': 0,
                        'stock_count': len(valid_results),
                        'profitable_stocks': 0,
                        'all_trade_profits': []
                    }
                    
                    # 收集每支股票的统计数据
                    for symbol, result in valid_results.items():
                        if 'stats' in result.iloc[-1]:
                            stats = result.iloc[-1]['stats']
                            total_stats['total_trades'] += stats['total_trades']
                            total_stats['profitable_trades'] += stats['profitable_trades']
                            total_stats['loss_trades'] += stats['loss_trades']
                            total_stats['total_holding_days'] += stats['avg_holding_days'] * stats['total_trades']
                            total_stats['total_profit'] += stats['final_profit']
                            total_stats['total_capital_usage'] += stats['max_capital_usage']
                            total_stats['total_win_rate'] += stats['win_rate']
                            
                            if stats['final_profit'] > 0:
                                total_stats['profitable_stocks'] += 1
                                
                            if hasattr(stats, 'trade_profits'):
                                total_stats['all_trade_profits'].extend(stats['trade_profits'])
                    
                    # 计算统计数据
                    avg_trades = total_stats['total_trades'] / total_stats['stock_count']
                    avg_holding_days = total_stats['total_holding_days'] / total_stats['total_trades'] if total_stats['total_trades'] > 0 else 0
                    avg_profit_per_trade = total_stats['total_profit'] / total_stats['total_trades'] if total_stats['total_trades'] > 0 else 0
                    median_profit_per_trade = np.median(total_stats['all_trade_profits']) if total_stats['all_trade_profits'] else 0
                    total_profit = total_stats['total_profit']
                    trade_win_rate = (total_stats['profitable_trades'] / total_stats['total_trades'] * 100) if total_stats['total_trades'] > 0 else 0
                    stock_win_rate = (total_stats['profitable_stocks'] / total_stats['stock_count'] * 100) if total_stats['stock_count'] > 0 else 0
                    
                    # 创建单个止盈点的结果DataFrame
                    result_df = pd.DataFrame([{
                        'take_profit_pct': take_profit_pct * 100,
                        'total_profit': total_profit,
                        'avg_profit_per_trade': avg_profit_per_trade,
                        'median_profit_per_trade': median_profit_per_trade,
                        'avg_holding_days': avg_holding_days,
                        'trade_win_rate': trade_win_rate,
                        'stock_win_rate': stock_win_rate,
                        'total_trades': total_stats['total_trades'],
                        'stock_count': total_stats['stock_count']
                    }])
                    
                    # 保存单个止盈点的结果
                    result_df.to_csv(temp_file, index=False)
                    print(f"保存止盈点 {take_profit_pct*100:.1f}% 的分析结果")
                    
                # 清理内存
                del valid_results
                del results
                if 'total_stats' in locals():
                    del total_stats
                
            except Exception as e:
                print(f"处理止盈点 {take_profit_pct*100:.1f}% 时出错: {str(e)}")
                continue
        
        # 合并所有结果
        print("\n合并所有分析结果...")
        all_results = []
        for take_profit_pct in profit_percentages:
            temp_file = f'results/temp/profit_{take_profit_pct*100:.1f}.csv'
            if os.path.exists(temp_file):
                try:
                    df = pd.read_csv(temp_file)
                    all_results.append(df)
                except Exception as e:
                    print(f"读取文件 {temp_file} 时出错: {str(e)}")
        
        if all_results:
            # 合并所有结果
            df = pd.concat(all_results, ignore_index=True)
            df = df.round({
                'take_profit_pct': 1,
                'total_profit': 0,
                'avg_profit_per_trade': 0,
                'median_profit_per_trade': 0,
                'avg_holding_days': 1,
                'trade_win_rate': 1,
                'stock_win_rate': 1,
                'total_trades': 0,
                'stock_count': 0
            })
            
            # 设置止盈比例为索引并排序
            df.set_index('take_profit_pct', inplace=True)
            df.sort_index(ascending=False, inplace=True)
            
            # 保存最终结果
            df.to_csv('results/profit_matrix_analysis.csv')
            
            # 打印结果
            print("\n止盈比例分析矩阵:")
            print("=" * 120)
            print(df.to_string())
            print("=" * 120)
            
            try:
                # 绘制并保存矩阵分析图表
                plt = self.plot_profit_matrix(df)
                plt.savefig('results/profit_matrix_analysis.png', bbox_inches='tight', dpi=300)
                plt.close()
            except Exception as e:
                print(f"绘制图表时出错: {str(e)}")
            
            return df
        else:
            print("没有找到任何分析结果")
            return pd.DataFrame()

def main():
    # 数据库连接配置
    db_config = {
        'host': '192.168.18.199',
        'port': 3306,
        'user': 'root',
        'password': 'Chlry#$.8',
        'database': 'db_gp_cj'
    }
    
    try:
        connection_string = f"mysql+pymysql://{db_config['user']}:{db_config['password']}@{db_config['host']}:{db_config['port']}/{db_config['database']}"
        
        # 创建results目录

        if not os.path.exists('results'):
            os.makedirs('results')
        
        # 创建分析器实例
        analyzer = StockAnalyzer(connection_string)
        
        # 设置分析参数
        start_date = '2022-05-05'
        end_date = '2023-08-28'
        stop_loss_pct = -0.99   # -99%止损
        pullback_entry_pct = -0.05  # -5%回调建仓
        
        # 运行止盈比例分析
        profit_matrix = analyzer.analyze_profit_matrix(
            start_date,
            end_date,
            stop_loss_pct,
            pullback_entry_pct
        )
        
        if not profit_matrix.empty:
            # 使用最优止盈比例进行完整分析
            take_profit_pct = profit_matrix['total_profit'].idxmax() / 100
            print(f"\n使用最优止盈比例 {take_profit_pct*100:.1f}% 运行详细分析")
            
            # 运行分析并只生成所需的图表
            results = analyzer.analyze_all_stocks(
                start_date,
                end_date,
                take_profit_pct,
                stop_loss_pct,
                pullback_entry_pct
            )
            
            # 过滤掉空结果
            valid_results = {symbol: result for symbol, result in results.items() if result is not None and not result.empty}
            
            if valid_results:
                # 只生成所需的两个图表
                plt, _ = analyzer.plot_all_stocks(valid_results)
                plt.savefig('results/all_stocks_analysis.png', bbox_inches='tight', dpi=300)
                plt.close()
        
    except Exception as e:
        print(f"程序运行出错: {str(e)}")
        raise

if __name__ == "__main__":
    main() 